﻿package
{
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	import flash.events.Event;
	
	public class Audio
	{
		// Declare vars here.
		private var snd:Sound;
		private var sndChannel:SoundChannel;
		private var tPause:int;
		private var timer:Timer;
		private var prompt:String;
		private var aBank:AudioBank;
		private var soundDone:Function;
		private var bMuted:Boolean;
		
		
		// Constructor.
		public function Audio()
		{
			// No sound as yet.
			snd        = null;
			sndChannel = null;
			soundDone  = null;
			prompt     = "";
			
			// No audio bank either.
			aBank = null;
			
			// Not muted.
			bMuted = false;
			
			// Set a default pause time.
			tPause = 500;
			
			// Create a timer.
			timer = new Timer(tPause, 1);
		}
		
		
		// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		// XXXX      SETTERS AND GETTERS      XXXX
		// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		

		public function set delay(t:Number):void
		// Purpose:  Sets the pause time (in seconds).
		{
			// Set the delay.
			tPause = int(t * 1000);
			
			// Create a new timer.
			timer.delay = tPause;
		}  // End: set delay(t:Number):void
		
		
		public function set bank(bnk:AudioBank):void
		// Purpose:  Sets the audio bank for this instance.
		{
			aBank = bnk;
		}  // End: set bank(bnk:AudioBank):void
		
		
		public function set onComplete(f:Function):void
		// Purpose:  Sets a function to be called when sound has finished playing.
		{
			soundDone = f;
		}  // End: set onComplete(f:Function):void
		
		
		public function set mute(bool:Boolean):void
		// Purpose:  Sets the mute state.
		{
			bMuted = bool;
		}  // End: mute(bool:Boolean):void
		
		
		// XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		// XXXX      FUNCTIONS      XXXX
		// XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		
		
		public function clear():void
		// Purpose:  Clears the prompt and sound.
		{
			prompt    = "";
			snd       = null;
			soundDone = null;
			
			// Clear the timer event.
			timer.removeEventListener(TimerEvent.TIMER, onTimedOut);
		}  // End: clear():void
		
		
		public function setSound(prmpt:String):void
		// Purpose:  Sets the current prompt and plays it.
		{
			// Set the prompt.
			prompt = prmpt;
			
			// If muted,
			if (bMuted)
			{
				// Don't play audio.
				return;
			}
			
			// After a delay, play the prompt.
			timer.addEventListener(TimerEvent.TIMER, onTimedOut, false, 0, true);
			
			// Start the timer.
			timer.start();
		}  // End: setSound(prmpt:String):void
		
		
		private function playPrompt():void
		// Purpose: Plays the prompt.
		{
			// Get the sound to play.
			snd = aBank.getAudio(prompt);
			
			// If the sound isn't in the bank,
			if (snd == null)
			{
				// Ignore it.
				return;
			}
			
			// Play it.
			sndChannel = snd.play();
			
			// If the user wants to know when it is done,
			if (soundDone != null)
			{
				// Listen for the end.
				sndChannel.addEventListener(Event.SOUND_COMPLETE, sndComplete, false, 0, true);
			}
		}  // End: playPrompt():void
		
		
		public function playSound(prmpt:String):void
		// Purpose:  Plays the audio requested immediately.
		{		
			// If muted,
			if (bMuted)
			{
				// Don't play audio.
				return;
			}
			// Get the sound to play.
			snd = aBank.getAudio(prmpt);
			
			// If the sound isn't in the bank,
			if (snd == null)
			{
				// Ignore it.
				return;
			}
			
			// Play it.
			snd.play();
		}  // End: playSound(prmpt:String):void
		
		
		public function replay():void
		// Purpose:  Replays the current prompt.
		{	
			// If muted,
			if (bMuted)
			{
				// Don't play audio.
				return;
			}
			playPrompt();
		}  // End: replay():void
		
		
		private function onTimedOut(evt:TimerEvent):void
		// Purpose:  Plays the prompt.
		{
			// Remove the listener.
			timer.removeEventListener(TimerEvent.TIMER, onTimedOut);
			
			// If the prompt has not been cleared in the meantime,
			if (prompt != "")
			{
				// Play the prompt.
				playPrompt();
			}
		}  // End: onTimedOut(evt:Timer);
		
		
		private function sndComplete(evt:Event):void
		// Purpose:  Performs the action that user wants done when the audio is completed.
		{
			// Remove the listener.
			sndChannel.removeEventListener(Event.SOUND_COMPLETE, sndComplete);
			
			// Perform the user's action.
			soundDone();
		}  // End: sndComplete(evt:Event):void
		
	}  // End: class Audio
}  // End: package